---------------------------------------------------------------------------------------------------
-- DEEDS (Digital Electronics Education and Design Suite)
-- VHDL Code generated on (12/21/2014, 8:31:19 PM)
--      by the Digital Circuit Simulator(d-DcS)
--      Ver. 1.80.100 (March 21, 2014)
-- Copyright(c)2002-2014 University of Genoa, Italy
--      Web Site:  http://www.esng.dibe.unige.it/deeds
---------------------------------------------------------------------------------------------------
-- Code compiled for: "DE2 Board" by Altera Corporation
-- Chip FPGA: Altera Cyclone(r) II EP2C35F672C6
-- Proprietary EDA Tool: Quartus(r) II (Ver > 11.0)
---------------------------------------------------------------------------------------------------


--------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

-- Clock Scaler and Debugger (Altera DE1, DE2 and DE2-115 version, master clock = 50 MHz)

ENTITY ClockScalerDebugger IS
	PORT(	iMClk: IN  std_logic;   -- Master Clock
			iH4:   IN  std_logic;   -- iH4..iH0 = "high" frequency selection
			iH3:   IN  std_logic;
			iH2:   IN  std_logic;
			iH1:   IN  std_logic;
			iH0:   IN  std_logic;
			iL3:   IN  std_logic;   -- iL3..iL0 = "low" frequency selection
			iL2:   IN  std_logic;   --                  and Button Modes
			iL1:   IN  std_logic;
			iL0:   IN  std_logic;
			iSwch: IN  std_logic;   -- Switch (low: iH<n> selection, high: iL<n> selection)
			iBut:  IN  std_logic;   -- Button for manual pulsed Clock, or Intruction Step By Step
			oSClk: OUT std_logic;   -- Output Clock
			oLed:  OUT std_logic; 	-- Slow "Clock Pulse" Led
         --
         -- DMC8 debugger interface ------------
 			inRes: IN  std_logic;	-- Debugger Reset
			iStep: IN  std_logic;	-- "Intruction Step by Step" input feedback
         oS0:   OUT std_logic;	-- CPU register selection (5 bits + nibble)
         oS1:   OUT std_logic;
         oS2:   OUT std_logic;
         oS3:   OUT std_logic;
         oS4:   OUT std_logic;
         oNi: 	 OUT std_logic;	-- Reg. nibble selection (0: low, 1: high)
         iR0:   IN  std_logic;   -- Register value return bus
         iR1:   IN  std_logic;
         iR2:   IN  std_logic;
         iR3:   IN  std_logic;
         --
         -- DE2 Display Interface -----
			D0_lcd: out std_logic;  -- Display Data (8 bits)
			D1_lcd: out std_logic;
			D2_lcd: out std_logic;
			D3_lcd: out std_logic;
			D4_lcd: out std_logic;
			D5_lcd: out std_logic;
			D6_lcd: out std_logic;
			D7_lcd: out std_logic;
			RW_lcd: out std_logic;  -- Control Signals (RS,RW,EN)
			EN_lcd: out std_logic;
			RS_lcd: out std_logic;
			ON_lcd: out std_logic;
			BL_lcd: out std_logic
			);
	END ClockScalerDebugger;

--------------------------------------------------------------------
ARCHITECTURE behavioral OF ClockScalerDebugger IS

-- "Auto Reset" shift register --------------------------------
SIGNAL	AutoReset_SHR:	unsigned( 15 downto 0 );
SIGNAL	nAutoReset: std_logic;

-- 10 mS Time Tick, x Input Debouncing ------------------------
--                               Clock 50 MHz --> 100Hz = 10 mS
SIGNAL	aButton_SHR:	unsigned( 2 downto 0 );
SIGNAL	aSwitch_SHR:	unsigned( 2 downto 0 );
SIGNAL	aButton: 		std_logic;
SIGNAL	aSwitch: 		std_logic;
SIGNAL	DebTimeTick:	std_logic;

-- Main Clock Scaler ------------------------------------------
constant	nBits:	integer:= 32;
constant	highBit:	integer:= nBits -1;
SIGNAL	aCount:  unsigned( highBit downto 0 );
SIGNAL	aCntMod: unsigned( highBit downto 0 );
SIGNAL	aCntMid: unsigned( highBit downto 0 );
SIGNAL 	aHIGH: unsigned( 4 downto 0 );
SIGNAL 	aLOW:  unsigned( 3 downto 0 );

-- Clock and LED Outputs --------------------------------------
SIGNAL 	InstrStepMode: std_logic;
SIGNAL 	InstrStepModeConf: std_logic;
SIGNAL	ManuaStepMode: std_logic;
SIGNAL 	ExeNextInstr:	std_logic;
--SIGNAL	StepPulse:		std_logic;
--SIGNAL	StepLED:			std_logic;
SIGNAL	DisplayDebugData: std_logic;
SIGNAL	DisplayDebugDataReg: std_logic;
SIGNAL	SplashOnReset: std_logic;


-- LCD Display 1602 -------------------------------------------
--
type sNames is(
	s_PowerOn, 			-- Power On Boot
	s_Set1, 				--	First "function set"
	s_Set2, 	  			--	Second "function set"
	s_Set3, 				--	Third "function set"
	s_Set8bit,			-- Last "function set" to 8 bit
	s_EntryModeSet,	-- Entry Mode Set
	s_DisplayOn, 		-- Set Display On
	s_ClearDisplay,	--	Clear Display
	s_InputWaiting,	-- Waiting for Input
	s_DisplayData, 	-- Write and Display data
	s_LineFeed 			-- Cursor to home, on the second line
   );
------------
CONSTANT	PowerOnDelay : integer := 2000;  --  20 ms (from 10 uS) [> 15 mS ]
CONSTANT	Set1_Delay   : integer :=  500;  --   5 ms (from 10 uS) [> 4.1 mS]
CONSTANT	Set2_Delay   : integer :=   50;	-- 0.5 ms (from 10 uS) [> 100 us]
CONSTANT	Set8bit_Delay: integer :=    6;	--  60 uS (from 10 uS) [> 39 us]
CONSTANT	Clear_Delay  : integer :=  200;  --   2 ms (from 10 uS) [> 1.53 mS]
CONSTANT	Small_delay  : integer :=    6;	--  60 uS (from 10 uS) [> 39 us, > 43 us]
--
CONSTANT	EN_begin     : integer := 1;  -- EN timing
CONSTANT	EN_end       : integer := 2;
------------
SIGNAL	DBus_lcd		: std_logic_vector(7 downto 0);
SIGNAL	RegSel		: std_logic_vector(5 downto 0);
SIGNAL	RegValue		: std_logic_vector(3 downto 0);
------------
SIGNAL	State			: sNames;
SIGNAL	DelayCnt		: integer range 0 to PowerOnDelay;
SIGNAL	NextCharPos	: integer range 0 to 32;
SIGNAL	LcdTick    	: std_logic;	-- Lcd Time Base (Time Tick)
SIGNAL	LcdBusy    	: std_logic;	-- '1' if Lcd busy
SIGNAL	LcdDataBuf	: std_logic_vector (7 downto 0);  -- Lcd Input Temp. Data Buffer
SIGNAL	LcdGetData 	: std_logic;                      -- Lcd Get buffered Temp. Data command
SIGNAL	LcdDataInput: std_logic_vector (7 downto 0);  -- Lcd Data Input
SIGNAL	LcdLoad		: std_logic;							 -- Lcd Load Data command
------------

-- Lcd: Page contents table (ASCII chars + codes) ------------------------
TYPE TLcd_Table is array (0 to 127) of std_logic_vector(7 downto 0);
CONSTANT Lcd_Table: TLcd_Table:= (
	--------------------------- "Splash" page 0 ------------
	--  "* DMC8 on FPGA *"
	--  "--------Deeds---"
	--
   000 => "00101010", -- '*'
   001 => "00100000", -- ' '
   002 => "01000100", -- 'D'
   003 => "01001101", -- 'M'
   004 => "01000011", -- 'C'
   005 => "00111000", -- '8'
   006 => "00100000", -- ' '
   007 => "01101111", -- 'o'
   008 => "01101110", -- 'n'
   009 => "00100000", -- ' '
   010 => "01000110", -- 'F'
   011 => "01010000", -- 'P'
   012 => "01000111", -- 'G'
   013 => "01000001", -- 'A'
   014 => "00100000", -- ' '
   015 => "00101010", -- '*'
	--
   016 => "00101101", -- '-'
   017 => "00101101", -- '-'
   018 => "00101101", -- '-'
   019 => "00101101", -- '-'.
   020 => "00101101", -- '-'
   021 => "00101101", -- '-'
   022 => "00101101", -- '-'
   023 => "00101101", -- '-'.
   024 => "01000100", -- 'D'
   025 => "01100101", -- 'e'
   026 => "01100101", -- 'e'
   027 => "01100100", -- 'd'
   028 => "01110011", -- 's'
   029 => "00101101", -- '-'
   030 => "00101101", -- '-'
   031 => "00101101", -- '-'
	--
	--------------------------- Debugger Display (page 1) --
	--	"PC=xxxx   Acc=xx"
	--	"F=00x00x00 Iff=x"
	--
   032 => "01010000", -- 'P'
   033 => "01000011", -- 'C'
   034 => "00111101", -- '='
   035 => "10000001", -- Code: PC.High, nibble High
   036 => "10000000", -- Code: PC.High, nibble Low
   037 => "10000011", -- Code: PC.Low, nibble High
   038 => "10000010", -- Code: PC.Low, nibble Low
   039 => "00100000", -- ' '
   040 => "00100000", -- ' '
   041 => "00100000", -- ' '
   042 => "01000001", -- 'A'
   043 => "01100011", -- 'c'
   044 => "01100011", -- 'c'
   045 => "00111101", -- '='
   046 => "10001001", -- Code: Acc, nibble High
   047 => "10001000", -- Code: Acc, nibble Low
	--
   048 => "01000110", -- 'F'
   049 => "00111101", -- '='
   050 => "10001010", -- Code: Flag C
   051 => "10001100", -- Code: Flag N
   052 => "01111000", -- 'x'
   053 => "10001110", -- Code: Flag P
	054 => "10010000", -- Code: Flag H
	055 => "01111000", -- 'x'
   056 => "10010010", -- Code: Flag Z
   057 => "10010100", -- Code: Flag S
   058 => "00100000", -- ' '
   059 => "01001001", -- 'I'
   060 => "01100110", -- 'f'
   061 => "01100110", -- 'f'
   062 => "00111101", -- '='
   063 => "10010110", -- Code: IFF
	--
	----------------------------------------------------- Debugger Display (page 2) --
	--	"PC=xxxx  BC=xxxx"
	--	"DE=xxxx  HL=xxxx"
	--
   064 => "01010000", -- 'P'
   065 => "01000011", -- 'C'
   066 => "00111101", -- '='
   067 => "10000001", -- Code: PC.High, nibble High
   068 => "10000000", -- Code: PC.High, nibble Low
   069 => "10000011", -- Code: PC.Low, nibble High
   070 => "10000010", -- Code: PC.Low, nibble Low
   071 => "00100000", -- ' '
   072 => "00100000", -- ' '
   073 => "01000010", -- 'B'
   074 => "01000011", -- 'C'
   075 => "00111101", -- '='
   076 => "10100001", -- Code: B, nibble High
   077 => "10100000", -- Code: B, nibble Low
   078 => "10100011", -- Code: C, nibble High
   079 => "10100010", -- Code: C, nibble Low
	--
   080 => "01000100", -- 'D'
   081 => "01000101", -- 'E'
   082 => "00111101", -- '='
   083 => "10100101", -- Code: D, nibble High
   084 => "10100100", -- Code: D, nibble Low
   085 => "10100111", -- Code: E, nibble High
   086 => "10100110", -- Code: E, nibble Low
   087 => "00100000", -- ' '
   088 => "00100000", -- ' '
   089 => "01001000", -- 'H'
   090 => "01001100", -- 'L'
   091 => "00111101", -- '='
   092 => "10101001", -- Code: H, nibble High
   093 => "10101000", -- Code: H, nibble Low
   094 => "10101011", -- Code: L, nibble High
   095 => "10101010", -- Code: L, nibble Low
	--
	----------------------------------------------------- Debugger Display (page 3) --
	--	"PC=xxxx  SP=xxxx"
	--	"IX=xxxx  IY=xxxx"
	--
   096 => "01010000", -- 'P'
   097 => "01000011", -- 'C'
   098 => "00111101", -- '='
   099 => "10000001", -- Code: PC.High, nibble High
	100 => "10000000", -- Code: PC.High, nibble Low
	101 => "10000011", -- Code: PC.Low, nibble High
	102 => "10000010", -- Code: PC.Low, nibble Low
	103 => "00100000", -- ' '
	104 => "00100000", -- ' '
	105 => "01010011", -- 'S'
	106 => "01010000", -- 'P'
	107 => "00111101", -- '='
	108 => "10000101", -- Code: SP.High, nibble High
	109 => "10000100", -- Code: SP.High, nibble Low
	110 => "10000111", -- Code: SP.Low, nibble Low
	111 => "10000110", -- Code: SP.Low, nibble Low
	--
	112 => "01001001", -- 'I'
	113 => "01011000", -- 'X'
	114 => "00111101", -- '='
	115 => "10101101", -- Code: IX.High, nibble High
	116 => "10101100", -- Code: IX.High, nibble Low
	117 => "10101111", -- Code: IX.Low, nibble Low
	118 => "10101110", -- Code: IX.Low, nibble Low
	119 => "00100000", -- ' '
	120 => "00100000", -- ' '
	121 => "01001001", -- 'I'
	122 => "01011001", -- 'Y'
	123 => "00111101", -- '='
	124 => "10111101", -- Code: IY.High, nibble High
	125 => "10111100", -- Code: IY.High, nibble Low
	126 => "10111111", -- Code: IY.Low, nibble Low
	127 => "10111110", -- Code: IY.Low, nibble Low
	--
	OTHERS=> "00100000"
	);

BEGIN

	------------------------------------------------------------
	-- Shift register to create a "Auto-Reset" signal
	------------------------------------------------------------
	Res: process( iMClk )
	begin
		if rising_edge( iMClk ) then
			if	(inRes = '0') then
				nAutoReset <= '0';
			else
				if (AutoReset_SHR = "1111111111111111") then
					nAutoReset <= '1';
				else nAutoReset <= '0';	-- '0' at power up
				end if;
			end if;
			AutoReset_SHR <= '1' & AutoReset_SHR(15 downto 1);
		end if;
	end process;


	------------------------------------------------------------
	-- Counter to generate some Time Ticks.
	-- 10 mS: 	used to debounce switches and button used for
	--				the "slow clock" and the "Instruction Step by
	--				Step" modes. It is used also as time base for
	--				the timing of button pressing repetitions.
	-- 10 uS:	used to clock Lcd main state machine
	------------------------------------------------------------
	TimeTicks: process( nAutoReset, iMClk )
		constant	lcdCountMax: integer:= 500-1;	 -- 10 uS
		constant	debCountMax: integer:= 1000-1; -- 10 mS
		variable lcdCount: integer range 0 to lcdCountMax;
		variable debCount: integer range 0 to debCountMax;
	begin
		if (nAutoReset = '0') then
			lcdCount := 0;
			debCount := 0;
			DebTimeTick <= '0';
        	LcdTick <= '0';
			--
		elsif rising_edge( iMClk ) then
			--
			if (lcdCount = lcdCountMax) then
         	lcdCount := 0;
	         LcdTick <= '1';  -- generate LCD 10 uS Time Tick
				--
				if (debCount = 0) then
					debCount := debCountMax;
					DebTimeTick <= '1'; -- generate Deb 10 mS Time Tick
				else
					debCount := debCount - 1;
					DebTimeTick <= '0';
				end if;
   	   else
      	   lcdCount := lcdCount + 1;
         	LcdTick <= '0';
				DebTimeTick <= '0';
	      end if;
		end if;
	end process;


	------------------------------------------------------------
	-- Shift registers for sincronize and debounce the signals:
	-- iSwch --> becomes  "aSwitch"
	-- iBut  --> becomes  "aButton"
	------------------------------------------------------------
	Debounce: process( nAutoReset, iMClk )
	begin
		if (nAutoReset = '0') then
			aSwitch_SHR <= "000";
			aSwitch <= '1';
			aButton_SHR <= "000";
			aButton <= '0';
		--
		elsif rising_edge( iMClk ) then
		  if (DebTimeTick = '1') then -- (10 mS period)

				--------- Switch ----------------------------------
				if    (aSwitch_SHR = "000") then  aSwitch <= '1';	-- '1' at Reset
				elsif (aSwitch_SHR = "111") then  aSwitch <= '0';
				end if;
				aSwitch_SHR <= (not iSwch) & aSwitch_SHR(2) & aSwitch_SHR(1);

			   --------- Button -----------------------------------
				if    (aButton_SHR = "000") then  aButton <= '0';	-- '0' at Reset
				elsif (aButton_SHR = "111") then  aButton <= '1';
				end if;
				aButton_SHR <= (not iBut) & aButton_SHR(2) & aButton_SHR(1);
				--
			end if;
		end if;
	end process;

	------------------------------------------------------------
	-- Scaled Clock: Frequency and Mode setting
	------------------------------------------------------------
	aHIGH <= iH4 & iH3 & iH2 & iH1 & iH0;
	aLOW  <= iL3 & iL2 & iL1 & iL0;

	CK_PERIOD: process( aSwitch, aHIGH, aLOW )
	begin
		InstrStepMode <= '0';
		ManuaStepMode <= '0';
		--
		if (aSwitch = '0') then  -- "normal mode" ---------------------
		---------------------------------------------------------------
			case aHIGH is
			---- 10 MHz ------------------------------------------------
			when "00000" =>   aCntMod <= TO_UNSIGNED( 5, nBits);
									aCntMid <= TO_UNSIGNED( 2, nBits);
			----  5 MHz ------------------------------------------------
			when "00001" =>   aCntMod <= TO_UNSIGNED(10, nBits);
									aCntMid <= TO_UNSIGNED( 5, nBits);
			----  2 MHz ------------------------------------------------
			when "00010" =>   aCntMod <= TO_UNSIGNED(25, nBits);
									aCntMid <= TO_UNSIGNED(12, nBits);
			----  1 MHz ------------------------------------------------
			when "00011" =>   aCntMod <= TO_UNSIGNED(50, nBits);
									aCntMid <= TO_UNSIGNED(25, nBits);
			---- 500 KHz -----------------------------------------------
			when "00100" =>   aCntMod <= TO_UNSIGNED(100, nBits);
									aCntMid <= TO_UNSIGNED( 50, nBits);
			---- 200 KHz -----------------------------------------------
			when "00101" =>   aCntMod <= TO_UNSIGNED(250, nBits);
									aCntMid <= TO_UNSIGNED(125, nBits);
			---- 100 KHz -----------------------------------------------
			when "00110" =>   aCntMod <= TO_UNSIGNED(500, nBits);
									aCntMid <= TO_UNSIGNED(250, nBits);
			---- 50 KHz ------------------------------------------------
			when "00111" =>   aCntMod <= TO_UNSIGNED(1000, nBits);
									aCntMid <= TO_UNSIGNED( 500, nBits);
			---- 20 KHz ------------------------------------------------
			when "01000" =>   aCntMod <= TO_UNSIGNED(2500, nBits);
									aCntMid <= TO_UNSIGNED(1250, nBits);
			---- 10 KHz ------------------------------------------------
			when "01001" =>   aCntMod <= TO_UNSIGNED(5000, nBits);
									aCntMid <= TO_UNSIGNED(2500, nBits);
			---- 5 KHz -------------------------------------------------
			when "01010" =>   aCntMod <= TO_UNSIGNED(10000, nBits);
									aCntMid <= TO_UNSIGNED( 5000, nBits);
			---- 2 KHz -------------------------------------------------
			when "01011" =>   aCntMod <= TO_UNSIGNED(25000, nBits);
									aCntMid <= TO_UNSIGNED(12500, nBits);
			---- 1 KHz -------------------------------------------------
			when "01100" =>   aCntMod <= TO_UNSIGNED(50000, nBits);
									aCntMid <= TO_UNSIGNED(25000, nBits);
			---- 500 Hz ------------------------------------------------
			when "01101" =>   aCntMod <= TO_UNSIGNED(100000, nBits);
									aCntMid <= TO_UNSIGNED( 50000, nBits);
			---- 200 Hz ------------------------------------------------
			when "01110" =>   aCntMod <= TO_UNSIGNED(250000, nBits);
									aCntMid <= TO_UNSIGNED(125000, nBits);
			---- 100 Hz ------------------------------------------------
			when "01111" =>   aCntMod <= TO_UNSIGNED(500000, nBits);
									aCntMid <= TO_UNSIGNED(250000, nBits);
			---- 50 Hz -------------------------------------------------
			when "10000" =>   aCntMod <= TO_UNSIGNED(1000000, nBits);
									aCntMid <= TO_UNSIGNED( 500000, nBits);
			---- 20 Hz -------------------------------------------------
			when "10001" =>   aCntMod <= TO_UNSIGNED(2500000, nBits);
									aCntMid <= TO_UNSIGNED(1250000, nBits);
			---- 10 Hz -------------------------------------------------
			when "10010" =>   aCntMod <= TO_UNSIGNED(5000000, nBits);
									aCntMid <= TO_UNSIGNED(2500000, nBits);
			---- 5 Hz --------------------------------------------------
			when "10011" =>   aCntMod <= TO_UNSIGNED(10000000, nBits);
									aCntMid <= TO_UNSIGNED( 5000000, nBits);
			---- 2 Hz --------------------------------------------------
			when "10100" =>   aCntMod <= TO_UNSIGNED(25000000, nBits);
									aCntMid <= TO_UNSIGNED(12500000, nBits);
			---- 1 Hz --------------------------------------------------
			when "10101" =>   aCntMod <= TO_UNSIGNED(50000000, nBits);
									aCntMid <= TO_UNSIGNED(25000000, nBits);
			--- if error: 1 Hz -----------------------------------------
			when others  =>   aCntMod <= TO_UNSIGNED(50000000, nBits);
									aCntMid <= TO_UNSIGNED(25000000, nBits);
			end case;

		else -- if (aSwitch = '1'.. "Instruction Step Modes" ----------
		---------------------------------------------------------------
			InstrStepMode <= '1';
         --
			case aLOW is
			---- 100 Instruction / Sec ---------------------------------
			when "0000" =>	aCntMod <= TO_UNSIGNED(500000, nBits);
								aCntMid <= TO_UNSIGNED(250000, nBits);
			---- 50 Instruction / Sec ----------------------------------
			when "0001" =>	aCntMod <= TO_UNSIGNED(1000000, nBits);
								aCntMid <= TO_UNSIGNED( 500000, nBits);
			---- 20 Instruction / Sec ----------------------------------
			when "0010" =>	aCntMod <= TO_UNSIGNED(2500000, nBits);
								aCntMid <= TO_UNSIGNED(1250000, nBits);
			---- 10 Instruction / Sec ----------------------------------
			when "0011" =>	aCntMod <= TO_UNSIGNED(5000000, nBits);
								aCntMid <= TO_UNSIGNED(2500000, nBits);
			---- 5 Instruction / Sec -----------------------------------
			when "0100" =>	aCntMod <= TO_UNSIGNED(10000000, nBits);
								aCntMid <= TO_UNSIGNED( 5000000, nBits);
			---- 2 Instruction / Sec -----------------------------------
			when "0101" =>	aCntMod <= TO_UNSIGNED(25000000, nBits);
								aCntMid <= TO_UNSIGNED(12500000, nBits);
			---- 1 Instruction / Sec -----------------------------------
			when "0110" =>	aCntMod <= TO_UNSIGNED(50000000, nBits);
								aCntMid <= TO_UNSIGNED(25000000, nBits);
			---- 0.5 Instruction / Sec ---------------------------------
			when "0111" =>	aCntMod <= TO_UNSIGNED(100000000, nBits);
								aCntMid <= TO_UNSIGNED( 50000000, nBits);
			---- 0.2 Instruction / Sec ---------------------------------
			when "1000" =>	aCntMod <= TO_UNSIGNED(250000000, nBits);
								aCntMid <= TO_UNSIGNED(125000000, nBits);
			---- 0.1 Instruction / Sec ----------------------------------
			when "1001" =>	aCntMod <= TO_UNSIGNED(500000000, nBits);
								aCntMid <= TO_UNSIGNED(250000000, nBits);
			--
			---- Manual Step by Step -----------------------------------
			when others => ManuaStepMode <= '1';
                        -- The following twos are here only to avoid latching
								aCntMod <= TO_UNSIGNED( 5000, nBits);  -- T= 100 uS
								aCntMid <= TO_UNSIGNED( 2500, nBits );
			end case;
		end if;
	end process;


	------------------------------------------------------------
	-- Clock Scaler Main Counter
	------------------------------------------------------------
	CNT: process( nAutoReset, iMClk )
      constant BurstCntMax: integer:= 5000-1;
      constant BurstCntMid: integer:= 2500;
      variable BurstCnt: integer range 0 to BurstCntMax;
      variable TickForNextInstr: boolean;
		variable StepInstruction: boolean;
		variable StartFirstInstruction: boolean;
		variable InstructionStepping: boolean;
		constant FlasherLedLow: integer := 3;
		constant FlasherLedHigh: integer := 5;
		variable Flasher: integer range 0 to FlasherLedHigh;
		constant FlashCounterEnd: integer := 18;
		variable FlashCounter: integer range 0 to FlashCounterEnd;
		variable FlashStepLed: boolean;
		--
	begin
		if (nAutoReset = '0') then
			---------------------------------------------------
			aCount <= (aCntMod - 1);
			BurstCnt := BurstCntMax;
			InstrStepModeConf <= '0';
			StartFirstInstruction := false;
			StepInstruction := false;
			FlashStepLed := false;
         TickForNextInstr:= false;
			oSClk <= '0';
			oLed <= '0';
			DisplayDebugData <= '0';
			---------------------------------------------------

		elsif rising_edge( iMClk ) then

      	if (InstrStepMode = '0') then

				--- Normal Cyclic Clock Mode ------------------------
				if ((aCount < 0) or (aCount >= aCntMod)) then
					aCount <= (aCntMod - 1); -- "Pseudo Reset"
				elsif (aCount = 0) then
					aCount <= (aCntMod - 1);
				else
					aCount <= (aCount - 1);
				end if;
				--
				if (aCount < aCntMid)	then
						oSClk <= '1'; 	 -- High if count is Low!
				else	oSClk <= '0';
				end if;
				oLed <= '0';          -- Led always Off
				--
				InstrStepModeConf <= '0';
				DisplayDebugData <= '0';
				StartFirstInstruction := false;
				StepInstruction := false;
				FlashStepLed := false;
				TickForNextInstr:= false;
     			BurstCnt := BurstCntMax;
			   ----------------------------------------------------

			else --(InstrStepMode = '1')

				-- Instruction Step Mode ---------------------------
            --   if in "Cyclic Step Mode" (= not in Manual Mode),
				--   and if the Splash Screen is not there....
				--   Count time to step the user-defined number
				--	  of Instructions per second:
				if (ManuaStepMode = '0') and (SplashOnReset = '0') then
            	if ((aCount < 0) or (aCount >= aCntMod)) then
						aCount <= (aCntMod - 1); -- "pseudo reset"
					elsif (aCount = 0) then
						aCount <= (aCntMod - 1);
						TickForNextInstr:= true; -- Order next instruction execution
					else
						aCount <= (aCount - 1);
					end if;
            end if;

				if (InstrStepModeConf = '0') then

				   --- Is first instruction step preparing --------
					InstrStepModeConf <= InstrStepMode;
				   StartFirstInstruction := true;
        			BurstCnt := BurstCntMax;
					oSClk <= '0';
					oLed <= '0';
					DisplayDebugData <= '0';

				else
	  				-- Is first instruction step requested? --------
					if StartFirstInstruction then
               	-- Yes, it is -------------------------------
						if (BurstCnt = 0) then
							BurstCnt := BurstCntMax;
							if (iStep = '1') then
								StartFirstInstruction := false;
								DisplayDebugData <= '1';
							end if;
						else
							BurstCnt := BurstCnt -1;
						end if;
						if (BurstCnt < BurstCntMid) then
								oSClk <= '1'; -- High if BurstCnt is low!
						else  oSClk <= '0';
						end if;
						oLed <= '0';  -- Led off during this phase

					elsif (DebTimeTick = '1') and -- every 10 mS --
                  	((ExeNextInstr = '1') or (TickForNextInstr)) then

						--- We have a request for another step ------
						StepInstruction := true;
						TickForNextInstr:= false;
						DisplayDebugData <= '0';
						BurstCnt := BurstCntMax;
						FlashStepLed := true;
						FlashCounter := 0;
						Flasher := 0;
						oSClk <= '0';
						oLed <= '1';
                  --
					else
						--- Subsequent Instruction Steps -----------------
						if StepInstruction then
							if (BurstCnt = 0) then
								BurstCnt := BurstCntMax;
								if (iStep = '1') then
									StepInstruction := false;
									DisplayDebugData <= '1';
								end if;
							else
								BurstCnt := BurstCnt -1;
							end if;
							--
							if (BurstCnt < BurstCntMid) then
								oSClk <= '1'; -- High if BurstCnt is low!
							else
								oSClk <= '0';
							end if;
						else
							BurstCnt := BurstCntMax;
							oSClk <= '0';
						end if;

						--- Flashing the Led to represent clock pulses --
						if FlashStepLed then
							if (DebTimeTick = '1') then

								-- Every 10 mS: ------------------------
								if (FlashCounter /= FlashCounterEnd) then
									if (Flasher = FlasherLedHigh) then
										oLed <= '1';
										Flasher:= 0;
									else
										if (Flasher = FlasherLedLow) then
											oLed <= '0';
										end if;
										Flasher:= Flasher + 1;
									end if;
									FlashCounter:= FlashCounter + 1;
								else
									oLed <= '0';
									FlashStepLed := false;
								end if;
							end if;
						end if;
						--------------------------------------------------
					end if;
				end if;
			end if;
		end if;
	end process;

	------------------------------------------------------------
	-- Button Step and Step Repetition Handler
	------------------------------------------------------------
	PULSE: process( nAutoReset, iMClk )
		constant IsTime: integer := 100;
		variable Stepper: integer range 0 to IsTime;
		variable Pulsing: boolean;
		constant PulserTimeCycle: integer := 70;
		variable Pulser: integer range 0 to PulserTimeCycle;
		--
	begin
		if (nAutoReset = '0') then
			ExeNextInstr <= '0';
			Stepper:= 0;
			Pulsing:= false;
			Pulser:= 0;

		elsif rising_edge( iMClk ) then
	      ------------------------------------------------------
			if (ManuaStepMode = '0') then
				ExeNextInstr <= '0';
				Stepper:= 0;
				Pulsing:= false;
				Pulser:= 0;
	         ---------------------------------------------------

			else --(ManuaStepMode = '1')
	         ----- Button Pulsed Mode --------------------------
				if (DebTimeTick = '1') then -- every 10 mS
					--
					if (aButton = '1')	then
               	if (not Pulsing) then
                  	if (Stepper < IsTime) then
								---------------------------------------
                        if (Stepper = 0) then
										ExeNextInstr <= '1';	-- order clock generator to start (Tw = 10 mS)
								else	ExeNextInstr <= '0';
								end if;
                     	Stepper:= Stepper + 1;

                     else --(Stepper = IsTime)
								---------------------------------------
                     	Pulsing:= true;
                     	Pulser:= PulserTimeCycle;
								ExeNextInstr <= '0';
                     end if;

                  else --(Pulsing)
                  	if (Pulser > 0) then
								---------------------------------------
                     	Pulser:= Pulser -1;
								ExeNextInstr <= '0';

                     else --(Pulser = 0)
								---------------------------------------
                     	Pulser:= PulserTimeCycle;
								ExeNextInstr <= '1';	-- order clock generator to start (Tw = 10 mS)
                     end if;
                  end if;
						--
					else -- (aButton = '0')
           			Stepper:= 0;
						Pulsing:= false;
						ExeNextInstr <= '0';
					end if;
            end if;
			end if;
		end if;
	end process;

---------------------------------------------------------------
-- LCD Display 1602 -------------------------------------------
--
   RW_lcd <= '0';		-- always in Write mode...
	BL_lcd <= '1';		-- LCD Back Light ON
	--
	D0_lcd <= DBus_lcd(0);  -- Internal Data Bus to Lcd Connections
	D1_lcd <= DBus_lcd(1);
	D2_lcd <= DBus_lcd(2);
	D3_lcd <= DBus_lcd(3);
	D4_lcd <= DBus_lcd(4);
	D5_lcd <= DBus_lcd(5);
	D6_lcd <= DBus_lcd(6);
	D7_lcd <= DBus_lcd(7);
	--
	oNi <= RegSel(0); 		-- Internal Register Selection Bus
	oS0 <= RegSel(1);
	oS1 <= RegSel(2);
	oS2 <= RegSel(3);
	oS3 <= RegSel(4);
	oS4 <= RegSel(5);
	--
   RegValue(0) <= iR0;   	-- Internal Register value return bus
   RegValue(1) <= iR1;
   RegValue(2) <= iR2;
   RegValue(3) <= iR3;


-- LCD Display 1602 -------------------------------------------
-- Read string / command table
--
	UpdateLCD: process( iMClk, nAutoReset )
		constant	PgCountMax: integer := 270-1; -- x 10 mS = 2.7 Seconds
		variable PgCount: integer range 0 to PgCountMax;
	   variable iTable : integer range 0 to 127;
		constant	iSetMax: integer := 96;
	   variable iSet : integer range 0 to 127;
   	variable SendState: boolean;
   	variable ClearDisplay: boolean;
   	variable Cleared: boolean;
		variable WaitingData: boolean;
		variable DisplayDebugUpEdge: boolean;
	begin

	   if (nAutoReset = '0') then
			PgCount := 0;
   	   iTable := 0;
			iSet := 0;
      	LcdGetData <= '0';
	      LcdDataBuf <= (others => '0');
   	   SendState := false;
			Cleared := false;
			ClearDisplay := false;
			WaitingData := false;
			DisplayDebugDataReg <= '0';
			DisplayDebugUpEdge := false;
			SplashOnReset <= '1';

	   elsif (rising_edge(iMClk)) then

		   -- Check /up edge/ of "DisplayDebugData" request:
			DisplayDebugUpEdge :=  (DisplayDebugData = '1') and (DisplayDebugDataReg = '0');
			DisplayDebugDataReg <= DisplayDebugData;

			if ((SplashOnReset = '0') and DisplayDebugUpEdge) or (InstrStepMode = '0') then
				PgCount := 0;
				iTable:= 0;
				iSet := 32;
				ClearDisplay := true;
			else
				-- Wait for a "visible" time... then change display page ---
				if (DebTimeTick = '1') then
					if (PgCount = PgCountMax) then
						PgCount := 0;
						iTable:= 0;
						if (iSet < iSetMax) then
								iSet := iSet + 32;
						else	iSet := 32;
						end if;
						ClearDisplay := true;
						SplashOnReset <= '0';
					else
						PgCount := PgCount + 1;
					end if;
				end if;
			end if;

			LcdGetData <= '0';
			if (DisplayDebugData = '0') or (ClearDisplay) then

				---- Clear Display ------------------------------------
				ClearDisplay := false;
				if (LcdBusy = '0') and (not Cleared) and (not SendState) then
					LcdDataBuf <= x"0C"; -- FormFeed => Clear Screen
					LcdGetData <= '1';
					iTable:= 0;
					WaitingData := false;
					Cleared := true;
					SendState := true;
				else
					SendState := false;
				end if;

			else --(DisplayDebugData = '1')

				---- Start Display ------------------------------------
				ClearDisplay := false;
				Cleared := false;
				if (LcdBusy = '0' and (not SendState) and iTable <= 31) then
					if (not WaitingData) then

						-- Read the current entry from the table: ------------
						if (Lcd_Table(iTable + iSet)(7) = '0') then

							-- It is simple ASCII char, send it:
							LcdDataBuf <= Lcd_Table(iTable + iSet);
							LcdGetData <= '1';
							iTable := iTable + 1;
							SendState := true;
							--
						else
						   -- It is Data code, ask the core the proper register value:
						   RegSel <= Lcd_Table(iTable + iSet)(5 downto 0);
							WaitingData := true;  -- Get data at the next clock edge
						end if;
						--
					else --(WaitingData)----------

						-- Data is there, convert it to an ASCII char:
						case RegValue is
							when "0000" => LcdDataBuf <= "00110000";  -- Hex '0'
							when "0001" => LcdDataBuf <= "00110001";  -- Hex '1'
							when "0010" => LcdDataBuf <= "00110010";  -- Hex '2'
							when "0011" => LcdDataBuf <= "00110011";  -- Hex '3'
							when "0100" => LcdDataBuf <= "00110100";  -- Hex '4'
							when "0101" => LcdDataBuf <= "00110101";  -- Hex '5'
							when "0110" => LcdDataBuf <= "00110110";  -- Hex '6'
							when "0111" => LcdDataBuf <= "00110111";  -- Hex '7'
							when "1000" => LcdDataBuf <= "00111000";  -- Hex '8'
							when "1001" => LcdDataBuf <= "00111001";  -- Hex '9'
							when "1010" => LcdDataBuf <= "01000001";  -- Hex 'A'
							when "1011" => LcdDataBuf <= "01000010";  -- Hex 'B'
							when "1100" => LcdDataBuf <= "01000011";  -- Hex 'C'
							when "1101" => LcdDataBuf <= "01000100";  -- Hex 'D'
							when "1110" => LcdDataBuf <= "01000101";  -- Hex 'E'
							when OTHERS => LcdDataBuf <= "01000110";  -- Hex 'F'
						end case;
						LcdGetData <= '1';
						iTable := iTable + 1;
						WaitingData := false;
						SendState := true;
						--
					end if;
				else
					SendState := false;
				end if;

			end if;
   	end if;
	end process;


-- LCD Display 1602 -------------------------------------------
-- Main State Maschine

	LCD: process( iMClk, nAutoReset )
	begin
		if (nAutoReset = '0') then
			ON_lcd		<= '0';
			State    	<= s_PowerOn;
			NextCharPos <= 0;
			DelayCnt 	<= 0;   -- up count
			EN_lcd   <= '0';
			RS_lcd   <= '1';
			LcdBusy	<= '1';
			LcdDataInput <= (others => '0');
			LcdLoad  <= '0';
			DBus_lcd	<= (others=>'0');

		elsif rising_edge(iMClk) then

			if (LcdGetData = '1') then
				LcdDataInput <= LcdDataBuf;
				LcdLoad <= '1';
				LcdBusy <= '1';
			end if;

			if LcdTick = '1' then
				case State is

			-- Wait... on power on ------------------------------------------ [ min. 15 mS ]
				when s_PowerOn =>
					ON_lcd  <= '1';
					EN_lcd  <= '0';
					LcdBusy <= '1';
					if (DelayCnt < PowerOnDelay) then  --(20 mS)
					  DelayCnt <= (DelayCnt + 1);
					  RS_lcd <= '1';
					else
					  DelayCnt <= 0;
					  RS_lcd <= '0';
					  State <= s_Set1;
					end if;

			-- First setup command ----------------------------------------- [ min. 4.1 mS ]
				 when s_Set1 =>
					DBus_lcd <= "00110000";  -- "00110000" = Set 8 bits (and wait)
					RS_lcd  <= '0';
					if 	(DelayCnt = EN_begin) then	EN_lcd <= '1';
					elsif	(DelayCnt = EN_end)	 then	EN_lcd <= '0';
					end if;
					if (DelayCnt < Set1_Delay) then  --(5 mS)
					  DelayCnt <= DelayCnt+1;
					else
					  DelayCnt <= 0;
					  State <= s_Set2;
					end if;

			-- Second setup command ---------------------------------------- [ min. 100 uS ]
				 when s_Set2 =>
					DBus_lcd <= "00110000";  -- "00110000" = Set 8 bits (and wait)
					RS_lcd  <= '0';
					if 	(DelayCnt = EN_begin) then	EN_lcd <= '1';
					elsif	(DelayCnt = EN_end)	 then	EN_lcd <= '0';
					end if;
					if (DelayCnt < Set2_Delay) then  --(0.5 ms)
					  DelayCnt <= DelayCnt+1;
					else
					  DelayCnt <= 0;
					  State <= s_Set3;
					end if;

			-- Third setup command ---------------------------------------- [ min. 39 uS ]
				 when s_Set3 =>
					DBus_lcd <= "00110000";  -- "00110000" = Set 8 bits (and wait)
					RS_lcd  <= '0';
					if 	(DelayCnt = EN_begin) then	EN_lcd <= '1';
					elsif	(DelayCnt = EN_end)	 then	EN_lcd <= '0';
					end if;
					if (DelayCnt < Small_delay) then
					  DelayCnt <= DelayCnt+1;
					else
					  DelayCnt <= 0;
					  State <= s_Set8bit;
					end if;

			-- Set Interface to 8 bit command ------------------------------ [ min. 39 uS ]
				 when s_Set8bit =>
					DBus_lcd <= "00111100";  -- "0011NFxx" = set 8 bit interface + 2 line display + font 5x10
					RS_lcd  <= '0';
					if 	(DelayCnt = EN_begin) then	EN_lcd <= '1';
					elsif	(DelayCnt = EN_end)	 then	EN_lcd <= '0';
					end if;
					if (DelayCnt < Set8bit_Delay) then
					  DelayCnt <= DelayCnt+1;
					else
					  DelayCnt <= 0;
					  State <= s_EntryModeSet;
					end if;

			-- Set Entry Mode ----------------------------------- [ min. 39 uS ]
				 when s_EntryModeSet =>
					DBus_lcd <= "00000110";  -- Incr. AC address after each character + no shift
					RS_lcd  <= '0';
					if 	(DelayCnt = EN_begin) then	EN_lcd <= '1';
					elsif	(DelayCnt = EN_end)	 then	EN_lcd <= '0';
					end if;
					if (DelayCnt < Small_delay) then
					  DelayCnt <= DelayCnt+1;
					else
					  DelayCnt <= 0;
					  State <= s_DisplayOn;
					end if;

			-- Set Display ON ----------------------------------- [ min. 39 uS ]
				 when s_DisplayOn =>
					DBus_lcd <= "00001100";  -- Display On + no cursor + no blinking
					RS_lcd  <= '0';
					if 	(DelayCnt = EN_begin) then	EN_lcd <= '1';
					elsif	(DelayCnt = EN_end)	 then	EN_lcd <= '0';
					end if;
					if (DelayCnt < Small_delay) then
					  DelayCnt <= DelayCnt+1;
					else
					  DelayCnt <= 0;
					  State <= s_ClearDisplay;
					end if;

			-- Clear Display ------------------------------------ [ min. 1.53 mS ]
				 when s_ClearDisplay =>
					DBus_lcd	<= "00000001";   -- Clear display memory + clear AC address
					RS_lcd   <= '0';
					NextCharPos <= 0;
					if 	(DelayCnt = EN_begin) then	EN_lcd <= '1';
					elsif	(DelayCnt = EN_end)	 then	EN_lcd <= '0';
					end if;
					if (DelayCnt < Clear_Delay) then
					  DelayCnt <= DelayCnt+1;
					else
					  DelayCnt <= 0;
					  State <= s_InputWaiting;
					end if;

			-- Waiting for Input Data ----------------------------------
				when s_InputWaiting =>
					if LcdLoad = '1' then
						LcdLoad <= '0';
						--
						if LcdDataInput = x"0C" then		-- FormFeed => Clear Screen
							State <= s_ClearDisplay;
						elsif (NextCharPos = 16) then	-- End of Line => Cursor to Home, Next Line
							State <= s_LineFeed;
						else								-- Get and display next Char
							State <= s_DisplayData;
						end if;
					else
						LcdBusy <= '0';
					end if;

			-- Display the character on the LCD ----------------- [ min. 43 uS ]
				 when s_DisplayData =>
					DBus_lcd <= LcdDataInput;
					RS_lcd <= '1';
					if 	(DelayCnt = EN_begin) then	EN_lcd <= '1';
					elsif	(DelayCnt = EN_end)	 then	EN_lcd <= '0';
					end if;
					if (DelayCnt < Small_delay) then
						DelayCnt <= DelayCnt + 1;
					else
						DelayCnt <= 0;
						NextCharPos <= NextCharPos + 1;  -- count next char position
						State <= s_InputWaiting;
					end if;

			-- Cursor to Home, on the second line ---------------  [ min. 39 uS ]
				 when s_LineFeed =>
					DBus_lcd <= "11000000";   -- set DDRAM addr to first char of line 2
					RS_lcd <= '0';
					if 	(DelayCnt = EN_begin) then	EN_lcd <= '1';
					elsif	(DelayCnt = EN_end)	 then	EN_lcd <= '0';
					end if;
					if (DelayCnt < Small_delay) then
						DelayCnt <= DelayCnt+1;
					else
						DelayCnt <= 0;
						DBus_lcd <= LcdDataInput;
						State <= s_DisplayData;
					end if;

				end case;
			end if;
		end if;
	end process;

END behavioral;

--------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;

-- Seven Segment Display Decoder

ENTITY SevenSegm_Decoder IS
  PORT( iD3:    IN  std_logic;  -- iD3 = MSB, iD0 = LSB
        iD2:    IN  std_logic;
        iD1:    IN  std_logic;
        iD0:    IN  std_logic;
        oHEX_a: OUT std_logic;  -- All "Active Low"
        oHEX_b: OUT std_logic;
        oHEX_c: OUT std_logic;
        oHEX_d: OUT std_logic;
        oHEX_e: OUT std_logic;
        oHEX_f: OUT std_logic;
        oHEX_g: OUT std_logic );
END SevenSegm_Decoder;

--------------------------------------------------------------------
ARCHITECTURE behavioral OF SevenSegm_Decoder IS
  SIGNAL HexNumber: std_logic_vector( 3 downto 0 );
BEGIN
  -- REM: All Segment are active Low
  HexNumber <= iD3 & iD2 & iD1 & iD0;
  with HexNumber select
    oHEX_a <= '0' when "0000", '1' when "0001", '0' when "0010", '0' when "0011",
              '1' when "0100", '0' when "0101", '0' when "0110", '0' when "0111",
              '0' when "1000", '0' when "1001", '0' when "1010", '1' when "1011",
              '0' when "1100", '1' when "1101", '0' when "1110", '0' when "1111",
              '1' when others;
  with HexNumber select
    oHEX_b <= '0' when "0000", '0' when "0001", '0' when "0010", '0' when "0011",
              '0' when "0100", '1' when "0101", '1' when "0110", '0' when "0111",
              '0' when "1000", '0' when "1001", '0' when "1010", '1' when "1011",
              '1' when "1100", '0' when "1101", '1' when "1110", '1' when "1111",
              '1' when others;
  with HexNumber select
    oHEX_c <= '0' when "0000", '0' when "0001", '1' when "0010", '0' when "0011",
              '0' when "0100", '0' when "0101", '0' when "0110", '0' when "0111",
              '0' when "1000", '0' when "1001", '0' when "1010", '0' when "1011",
              '1' when "1100", '0' when "1101", '1' when "1110", '1' when "1111",
              '1' when others;
  with HexNumber select
    oHEX_d <= '0' when "0000", '1' when "0001", '0' when "0010", '0' when "0011",
              '1' when "0100", '0' when "0101", '0' when "0110", '1' when "0111",
              '0' when "1000", '0' when "1001", '1' when "1010", '0' when "1011",
              '0' when "1100", '0' when "1101", '0' when "1110", '1' when "1111",
              '1' when others;
  with HexNumber select
    oHEX_e <= '0' when "0000", '1' when "0001", '0' when "0010", '1' when "0011",
              '1' when "0100", '1' when "0101", '0' when "0110", '1' when "0111",
              '0' when "1000", '1' when "1001", '0' when "1010", '0' when "1011",
              '0' when "1100", '0' when "1101", '0' when "1110", '0' when "1111",
              '1' when others;
  with HexNumber select
    oHEX_f <= '0' when "0000", '1' when "0001", '1' when "0010", '1' when "0011",
              '0' when "0100", '0' when "0101", '0' when "0110", '1' when "0111",
              '0' when "1000", '0' when "1001", '0' when "1010", '0' when "1011",
              '0' when "1100", '1' when "1101", '0' when "1110", '0' when "1111",
              '1' when others;
  with HexNumber select
    oHEX_g <= '1' when "0000", '1' when "0001", '0' when "0010", '0' when "0011",
              '0' when "0100", '0' when "0101", '0' when "0110", '1' when "0111",
              '0' when "1000", '0' when "1001", '0' when "1010", '0' when "1011",
              '1' when "1100", '0' when "1101", '0' when "1110", '0' when "1111",
              '1' when others;
END behavioral;

